"""
Created on 26 mars 2018

@author: Stephane Tonneau

"""

import logging
from pidclient.base_utils import LoggingMethod


NODENAME = "file"

# TEMPLATES = ["/tmp_epod/pid/logs/loggingEntry_{0}", "/tmp/loggingEntry_{0}"]
TEMPLATES = ["./loggingEntry_{0}"]


class FileLogging(LoggingMethod):
    """Class that encapsulates the storing of objects in a file
    for logging purposes.
    """

    py_log = logging.getLogger(__name__)

    def __init__(self, **parameters):
        """Initialisator of the class

        :param configuration: the general logging configuration
        :type configuration: a python configuration object
        """
        super(FileLogging, self).__init__(**parameters)
        es_filename = self.get_config(NODENAME, "filename")
        if es_filename is None:
            self.templates = TEMPLATES
        else:
            self.templates = []
            templates = es_filename.split(",")
            for value in templates:
                self.templates.append(value+"_{0}")
        self.file_position = 0
        self.active_template = 0

    def close(self):
        """Close the logging instance"""
        pass

    def process_log(self, entry, meta_information):
        """Method to insert a json representation of both an object
        and its metadata in a file

        :param entry: the standard information to log
        :type entry: PidEntry
        :param meta_information: meta information about the information to log
        :type meta_information: Metainformation
        """
        doc = entry.to_json()

        filename = self.templates[self.active_template].format(meta_information.uid)
        nbr = len(self.templates)
        while nbr > 0:
            try:
                with open(filename, "w") as text_file:
                    text_file.write(str(doc))
                break
            except Exception:
                self.active_template += 1
                if self.active_template >= len(self.templates):
                    self.active_template = 0
                filename = self.templates[
                    self.active_template].format(meta_information.uid)
                nbr -= 1
        if nbr <= 0:
            self.py_log.error("Unable to write the logging entry to file")

        self.file_position += 1
